import React from 'react';
import PropTypes from 'prop-types';
import { withGoogleMap, GoogleMap, InfoWindow, Marker } from 'react-google-maps';
import withScriptjs from 'react-google-maps/lib/async/withScriptjs';
import { LoadingIndicator } from '6thman-shared';

/*
	NOTE: This whole componenet is a pain in the ass. The Google Maps API script tag must be loaded before
	the component can be rendered. That is why the MapWrapper uses withScriptJs. There is no way to just
	`npm install` and import the Google Maps API (bc it is not open source?). So this MapWrapper loads
	a script tag onto the page, then builds/renders the component. We could get around using the withScriptjs,
	but we would have to manually add the script tag...this seems like best solution bc it only loads the
	tag on the page that needs it.
*/

const MapWrapper = withScriptjs(withGoogleMap((props) => (
  <GoogleMap
  	//defaultZoom={props.zoom} commented out bc auto-zoom
  	center={props.center}
  	ref={(map) => {
    const offset = ((screen.width * .4166666) / 2) * -1; // Half width of the #dashboardContent, then negative
    if (!props.center.lat || !props.center.lng) {
				// Center the map based on marker cluster if no specific center
      const bounds  = new window.google.maps.LatLngBounds();
      props.markers.forEach((marker) => {bounds.extend(marker.position);});
      map && map.fitBounds(bounds); // auto zoom
      map && map.panToBounds(bounds); // auto center
      map && map.panBy(offset, 0);
    } else {
      map && map.panTo(props.center);
      map && map.panBy(offset, 0);
    }
  }}
  	defaultOptions={{
    fullscreenControl: true,
    mapTypeControlOptions: {position: window.google.maps.ControlPosition.TOP_RIGHT}
  }}
  >
    {props.markers.map((marker, index) => (
      <Marker
        key={index}
        position={marker.position}
        onClick={() => props.onMarkerClick(marker)}
        label={marker.label}
        icon={marker.icon}
      >
       {/*
        onMouseOver={() => props.onMarkerClick(marker)}
        onMouseOut={() => props.onMarkerClose(marker)}
				*/}
        {/*
          Show info window only if the 'showInfo' key of the marker is true.
          That is, when the Marker pin has been clicked and 'onCloseClick' has been
          Successfully fired.
        */}
        {marker.showInfo && (
          <InfoWindow onCloseClick={() => props.onMarkerClose(marker)}>
            <div>{marker.infoContent}</div>
          </InfoWindow>
        )}
      </Marker>
    ))}
  </GoogleMap>
)));

import './ActiveJobsMap.css';
class ActiveJobsMap extends React.Component {
  constructor(props) {
    super(props);
    this.icons = {
      StageMilestone: `${process.env.PUBLIC_URL}/truck_blue.png`,
      StageTravel: `${process.env.PUBLIC_URL}/truck_grey.png`,
      StageWork: `${process.env.PUBLIC_URL}/truck_green.png`
    };
    this.state = {
      markers: this.createMarkers(this.props.jobs)
    };
  }

  componentWillReceiveProps(nextProps) {
    this.setState({markers: this.createMarkers(nextProps.jobs)});
  }

  createMarkers(jobs) {
    let markers = jobs.map((job) => {
      let label = job.customer.last_name;
      label = label.length <= 9 ? label : `${label.substring(0, 7)}...`;
      return {
        position: {lat: job.current_geo.coords.latitude, lng: job.current_geo.coords.longitude},
        showInfo: false,
        infoContent: `${job.user} | ${job.crew}`,
        label: label,
        icon: this.icons[job.currentStage.content_template_model]
      };
    });
    return markers;
  }

  // Toggle to 'true' to show InfoWindow and re-renders component
  handleMarkerClick(targetMarker) {
    this.setState({
      markers: this.state.markers.map((marker) => {
        if (marker === targetMarker) {
          return { ...marker, showInfo: true };
        }
        return marker;
      }),
    });
  }

  handleMarkerClose(targetMarker) {
    this.setState({
      markers: this.state.markers.map(marker => {
        if (marker === targetMarker) {
          return { ...marker, showInfo: false };
        }
        return marker;
      }),
    });
  }

  render() {
    return (
			<div id="ActiveJobsMap">
				<MapWrapper
					googleMapURL={`https://maps.googleapis.com/maps/api/js?v=3.exp&libraries=geometry,drawing,places&key=${process.env.REACT_APP_GOOGLE_MAPS_KEY}`}
					loadingElement={<LoadingIndicator />}
					containerElement={<div className="map" />}
					mapElement={<div className="map" />}
					center={this.props.center}
					//zoom={this.props.zoom} commented out bc auto-zoom
					markers={this.state.markers}
					onMarkerClick={this.handleMarkerClick.bind(this)}
					onMarkerClose={this.handleMarkerClose.bind(this)}
				/>
			</div>
    );
  }
}

ActiveJobsMap.propTypes = {
  jobs: PropTypes.array.isRequired,
  center: PropTypes.object
	//zoom: PropTypes.number commented out bc auto-zoom
};

export default ActiveJobsMap;
