import React from 'react';
import PropTypes from 'prop-types';
import { withGoogleMap, GoogleMap, InfoWindow, Marker } from 'react-google-maps';
import withScriptjs from 'react-google-maps/lib/async/withScriptjs';
import config from '../../../../../config/client';

import LoadingIndicator from '../../../shared/components/LoadingIndicator/LoadingIndicator.jsx';

/*
	NOTE: This whole componenet is a pain in the ass. The Google Maps API script tag must be loaded before
	the component can be rendered. That is why the MapWrapper uses withScriptJs. There is no way to just
	`npm install` and import the Google Maps API (bc it is not open source?). So this MapWrapper loads
	a script tag onto the page, then builds/renders the component. We could get around using the withScriptjs,
	but we would have to manually add the script tag...this seems like best solution bc it only loads the
	tag on the page that needs it.
*/

const { google } = window;

const MapWrapper = withScriptjs(withGoogleMap((props) => (
  <GoogleMap
  	defaultZoom={props.zoom}
  	center={props.center}
  	defaultOptions={{
    fullscreenControl: true,
    mapTypeControlOptions: {position: google.maps.ControlPosition.TOP_RIGHT}
  }}
  >
    {props.markers.map((marker, index) => (
      <Marker
        key={index}
        position={marker.position}
        onClick={() => props.onMarkerClick(marker)}
        label={marker.label}
        icon={marker.icon}
      >
       {/*
        onMouseOver={() => props.onMarkerClick(marker)}
        onMouseOut={() => props.onMarkerClose(marker)}
				*/}
        {/*
          Show info window only if the 'showInfo' key of the marker is true.
          That is, when the Marker pin has been clicked and 'onCloseClick' has been
          Successfully fired.
        */}
        {marker.showInfo && (
          <InfoWindow onCloseClick={() => props.onMarkerClose(marker)}>
            <div>{marker.infoContent}</div>
          </InfoWindow>
        )}
      </Marker>
    ))}
  </GoogleMap>
)));

import './JobMap.css';
class JobMap extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      markers: this.createMarkers(this.props.locations)
    };
  }

  createMarkers(locations) {
    let markers = locations.map((job) => (
			// TODO: Add some way to make icons variable, maybe an icon for what stage type they are in
      {
        position: {lat: job.current_geo.coords.latitude, lng: job.current_geo.coords.longitude},
        showInfo: false,
        infoContent: `${job.user} | ${job.crew}`,
        label: `${job.ref_id}`,
        icon: 'https://developers.google.com/maps/documentation/javascript/examples/full/images/beachflag.png'
      }
		));
    return markers;
  }

  // Toggle to 'true' to show InfoWindow and re-renders component
  handleMarkerClick(targetMarker) {
    this.setState({
      markers: this.state.markers.map((marker) => {
        if (marker === targetMarker) {
          return { ...marker, showInfo: true };
        }
        return marker;
      }),
    });
  }

  handleMarkerClose(targetMarker) {
    this.setState({
      markers: this.state.markers.map(marker => {
        if (marker === targetMarker) {
          return { ...marker, showInfo: false };
        }
        return marker;
      }),
    });
  }

  render() {
    return (
			<div id="JobMap">
				<MapWrapper
					googleMapURL={`https://maps.googleapis.com/maps/api/js?v=3.exp&libraries=geometry,drawing,places&key=${config.google.MAPS_KEY}`}
					loadingElement={<LoadingIndicator />}
					containerElement={<div className="map" />}
					mapElement={<div className="map" />}
					center={this.props.center}
					zoom={this.props.zoom}
					markers={this.state.markers}
					onMarkerClick={this.handleMarkerClick.bind(this)}
					onMarkerClose={this.handleMarkerClose.bind(this)}
				/>
			</div>
    );
  }
}

JobMap.propTypes = {
  locations: PropTypes.array.isRequired,
  center: PropTypes.object.isRequired,
  zoom: PropTypes.number.isRequired
};

export default JobMap;
