import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import { Timeline }  from '6thman-shared';

const earlierDate = (dateString1, dateString2) => {
  if (!dateString1) { return dateString2; }
  if (!dateString2) { return dateString1; }
  return new Date(dateString1) < new Date(dateString2) ? dateString1 : dateString2;
};

const laterDate = (dateString1, dateString2) => {
  if (!dateString1) { return dateString2; }
  if (!dateString2) { return dateString1; }
  return new Date(dateString1) > new Date(dateString2) ? dateString1 : dateString2;
};

const getEstimate = (startDate, estimateHours) => {
  return moment(startDate).add(estimateHours, 'hours', true).toISOString();
};

const jobIsInactive = (job) => (job.status === 'inactive');
const jobIsActive = (job) => (job.status === 'active');
//const jobIsComplete = (job) => (job.status === 'complete');

const stageIsInactive = (stage) => (!stage.start.time && !stage.end.time);
const stageIsActive = (stage) => (stage.start.time && !stage.end.time);
const stageIsComplete = (stage) => (stage.start.time && stage.end.time);

import './JobTimeline.css';
class JobTimeline extends React.Component {
//	constructor(props) {
//		super(props);
//	}

  createStatefulStages(job) {

		// Determine the original start times of each stage
    const stages = job.stages;
    const original_scheduled_times = stages.reduce((scheduled_times, stage, index) => {
      const prevIndex = index - 1;
      if (index === 0) {
				// this is the first stage and is scheduled to start at job.scheduled_time
        scheduled_times[index] = job.scheduled_time;
      } else {
        scheduled_times[index] = getEstimate(scheduled_times[prevIndex],
																						 stages[prevIndex].original_estimate);
      }
      return scheduled_times;
    }, {});

		// Determine the adjusted start times of each stage
    const adjusted_scheduled_times = stages.reduce((scheduled_times, stage, index) => {
      const prevIndex = index - 1;
      if (index === 0) {
        scheduled_times[index] = jobIsInactive(job) ? job.scheduled_time : stage.start.time;
      } else if (stageIsComplete(stage) || stageIsActive(stage)) {
				// the stage is complete/active, it has an actual start time
        scheduled_times[index] = stage.start.time;
      } else {
				// the stage is inactive,
				// each stage's start time is the same as the last stage's end, so use that if possible
        const prevStage = stages[prevIndex];
        const estimate = prevStage.adjusted_estimate ? prevStage.adjusted_estimate :
																												prevStage.original_estimate;
        const prevStageEnd = prevStage.end.time ? prevStage.end.time :
																									getEstimate(scheduled_times[prevIndex], estimate);
        scheduled_times[index] = prevStageEnd;
      }
      return scheduled_times;
    }, {});

		// Map the stages into items that can be used to get scale and tasks
    return job.stages.map((stage, index) => {
      if (stageIsInactive(stage)) {
        stage.status = 'inactive';
        stage.adjusted_estimate = stage.original_estimate;
      }	else if (stageIsActive(stage)) {
        stage.status = 'active';
        stage.adjusted_estimate = stage.adjusted_estimate ? stage.adjusted_estimate :
																														stage.original_estimate;
      } else if (stageIsComplete(stage)) {
        stage.status = 'completed';
        stage.adjusted_estimate = moment(stage.end.time).diff(stage.start.time, 'hours', true);
      }
      stage.scheduled_time = original_scheduled_times[index];
      stage.scheduled_end = getEstimate(stage.scheduled_time, stage.original_estimate);
      stage.adjusted_scheduled_time = adjusted_scheduled_times[index];
      stage.adjusted_scheduled_end = getEstimate(stage.adjusted_scheduled_time,
																								 stage.adjusted_estimate);
      return stage;
    });
  }

  getScale(stages) {
    const scale =  stages.reduce((scale, stage) => {

      const scheduledStart = earlierDate(stage.scheduled_time, stage.adjusted_scheduled_time),
        earlierStart = earlierDate(stage.start.time, scheduledStart);

      const scheduledEnd = laterDate(stage.scheduled_end, stage.adjusted_scheduled_end),
        laterEnd = laterDate(stage.end.time, scheduledEnd);

      scale.start = earlierDate(scale.start, earlierStart);
      scale.end = laterDate(scale.end, laterEnd);

      return scale;

    }, {start: '', end: ''});
		// Round to nearest hours and create total diff
    scale.start = moment(scale.start).startOf('hour');
    scale.end = moment(scale.end).endOf('hour');
    scale.total = moment(scale.end).diff(scale.start, 'hours');
    return scale;
  }

  getTasks(job, stages) {
    return stages.map((stage, index) => {
      let mappedStage = {
        id: index,
        status: stage.status,
        label: stage.title,
        original: {
          start: stage.scheduled_time,
          end: stage.scheduled_end,
          duration: stage.original_estimate,
        },
        adjusted: {}
      };
      if (stage.status === 'inactive') {
        mappedStage.adjusted = mappedStage.original;
        if (jobIsActive(job)) {
					// if the job is started, estimate
          mappedStage.adjusted = {
            start: stage.adjusted_scheduled_time,
            end: stage.adjusted_scheduled_end,
            duration: stage.adjusted_estimate,
          };
        }
        return mappedStage;
      } else if (stage.status === 'active') {
        mappedStage.adjusted = {
          start: stage.start.time,
          end: stage.adjusted_scheduled_end,
          duration: stage.adjusted_estimate
        };
        return mappedStage;
      } else {
        mappedStage.adjusted = {
          start: stage.start.time,
          end: stage.end.time,
          duration: moment(stage.end.time).diff(stage.start.time, 'hours', true)
        };
        return mappedStage;
      }
    });
  }

  render() {
    const stages = this.createStatefulStages(this.props.job);
    const scale = this.getScale(stages);
    const tasks = this.getTasks(this.props.job, stages);
    return (
			<div id="jobTimeline">
				<Timeline tasks={tasks} scale={scale} showOriginal={true}/>
			</div>
    );
  }

}

JobTimeline.propTypes = {
  job: PropTypes.object.isRequired
};

export default JobTimeline;
